#include "stdafx.h"


#include "adapIntgrtn.h"

using namespace adapInt;

namespace calc_t {

	double df;
	double t_kernel( double x ){
		double t = 1.0 / Math::Pow( 1 + x * x / df, (df + 1.0)/2.0 );
		return t;
	}

	double ga;

	double k_gamma_tr( double t ){
		double v = Math::Exp( (ga-1.0) * Math::Log(-Math::Log(t)) );
		return v;
	}

	double k_gamma( double y ){
		double t = Math::Exp( (ga-1.0) * Math::Log(y) ) * Math::Exp(-y);
		return t;
	}

	double gamma_r( double r ){
		ga = r;
		double c = Math::Exp(-1.0);
		AdaptiveGL gl;
		double v1 = gl.integral( 0.0, 1.0, k_gamma );
		double v2 = gl.integral( 0.0, c, k_gamma_tr );
		return v1 + v2;
	}

	double Cum_t_distribution( double t ) {
		if ( t == 0.0 ) {
			return 0.5;
		} else { if ( t > 0.0 ) {
				AdaptiveGL gl;
				double v = gl.integral( 0.0, t, t_kernel );
				v = 0.5 + gamma_r( (df + 1.0)/2.0) * v / (Math::Sqrt(Math::PI * df) * gamma_r(df/2.0));
				return v;
			} else {
				AdaptiveGL gl;
				double v = gl.integral( t, 0.0, t_kernel );
				v = 0.5 - gamma_r( (df + 1.0) / 2.0 ) * v / (Math::Sqrt(Math::PI * df) * gamma_r(df/2.0));
				return v;
			};
		}
	}

	double t_density( double t ){
		double v = Math::Sqrt(Math::PI * df) * gamma_r(df/2.0) * Math::Pow( 1 + t * t / df, (df + 1.0)/2.0 );
		return gamma_r((df + 1.0)/2.0) / v;
	}

//     Bisection@ɂ鍪̌vZ    

	void Bisection( double (*f)(double),
					double s,					 //   f֐̂Ƃ s > 0.0
					double c,					 //   ɑ΂֐l
					double L_b, double U_b,		 //   ̑݋Ԃ̉Ə
					double & Root,				 //     f(Root) = c
					double acc ){
		if ( (f(L_b) - c) * (f(U_b) - c) > 0.0 ){
			throw  gcnew Exception("Ɖ̑gݍ킹sK؂łB");
		}
		do {
			double m = 0.5 * (L_b + U_b);
			double v = f(m);
			if ( s * (v - c) > 0.0 ) {
				U_b = m;
			} else {
				L_b = m;
			}
		} while ( (Math::Abs(U_b - L_b) >= acc * Math::Abs(U_b))
					&&
				  ( (Math::Abs(U_b) + Math::Abs(L_b)) >= 1.0e-14)
				);
		Root = 0.5 * (L_b + U_b);
	}


//     Newton@ɂ鍪̌vZ   

	void Newton( double & x,				//  ̏lA߂l͍ f(x)=c
				 double c,					//  ֐l
				 double (*f)( double ),		//  ֐
				 double (*df)( double ),	//  ֐
				 double acc ){
		 int NStep = 0;
		 int ck = 0;
		 double x0;
		 do {
			 NStep++;
			 if (NStep > 20) {			//  20ȏJԂ̂Ƃ
				 ck = 1;
				 break;
			 };
			 x0 = x;
			 x = x0 + (c - f(x0)) / df(x0);
		 } while ( (Math::Abs(x - x0) >= (acc * (Math::Abs(x) + Math::Abs(x0))))
					 &&
					 ( (Math::Abs(x) + Math::Abs(x0)) >= 1.0e-14 ) 
			      );

		 if (ck == 0) return;

         //   Bisection@ŌvZ

		 double L_b = 0.0;
		 do {
				L_b -= 1.0;
		 } while ( f(L_b) >= c );
		 double U_b = 0.0;
		 do {
				U_b += 1.0;
		 } while ( f(U_b) <= c );
		 Bisection( f, 1.0, c, L_b, U_b, x, acc );
		 return;
	}


	//
	//		plɑ΂t̒l߂   p = Pr(x <= t)
	//
	double calc_t_for_p( double p ){
		double t = 0.0;
		Newton( t, p, Cum_t_distribution, t_density, 1.0e-12 );
		return t;
	}

}
